# Generated by Django 2.0.5 on 2018-05-11 13:30

import django.contrib.postgres.fields.jsonb
import django.core.serializers.json
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('elasticsearch_django', '0006_add_encoder_JSONField_kwarg'),
    ]

    operations = [
        migrations.AlterField(
            model_name='searchquery',
            name='hits',
            field=django.contrib.postgres.fields.jsonb.JSONField(encoder=django.core.serializers.json.DjangoJSONEncoder, help_text='The list of meta info for each of the query matches returned.'),
        ),
        migrations.AlterField(
            model_name='searchquery',
            name='query',
            field=django.contrib.postgres.fields.jsonb.JSONField(encoder=django.core.serializers.json.DjangoJSONEncoder, help_text='The raw ElasticSearch DSL query.'),
        ),
    ]



from unittest import mock

from django.core.exceptions import ImproperlyConfigured
from django.test import TestCase

from ..apps import (
    ElasticAppConfig,
    _validate_config,
    _validate_model,
    _validate_mapping,
    _connect_signals,
    _on_model_delete,
    _on_model_save,
    _update_search_index,
)
from .. import tests
from ..models import SearchDocumentMixin
from ..tests import TestModel


class SearchAppsConfigTests(TestCase):

    """Tests for the apps module ready function."""

    @mock.patch('elasticsearch_django.apps.settings.get_setting')
    @mock.patch('elasticsearch_django.apps._validate_config')
    @mock.patch('elasticsearch_django.apps._connect_signals')
    def test_ready(self, mock_signals, mock_config, mock_setting):
        """Test the AppConfig.ready method."""
        mock_setting.return_value = True  # auto-sync
        config = ElasticAppConfig('foo_bar', tests)
        config.ready()
        mock_config.assert_called_once_with(mock_setting.return_value)
        mock_signals.assert_called_once_with()

        mock_setting.return_value = False  # auto-sync
        mock_signals.reset_mock()
        mock_config.reset_mock()
        config.ready()
        mock_config.assert_called_once_with(mock_setting.return_value)
        mock_signals.assert_not_called()


class SearchAppsValidationTests(TestCase):

    """Tests for the apps module validation functions."""

    def test__validate_model(self):
        """Test _validate_model function."""
        # 1. model doesn't implement as_search_document
        with mock.patch('elasticsearch_django.tests.test_apps.TestModel') as tm:
            del tm.as_search_document
            self.assertRaises(ImproperlyConfigured, _validate_model, tm)

        # 2. model.objects doesn't implement get_search_queryset
        with mock.patch('elasticsearch_django.tests.test_apps.TestModel') as tm:
            del tm.objects.get_search_queryset
            self.assertRaises(ImproperlyConfigured, _validate_model, tm)

        # model should pass
        with mock.patch('elasticsearch_django.tests.test_apps.TestModel') as tm:
            _validate_model(tm)

    @mock.patch('elasticsearch_django.apps.settings')
    def test__validate_mapping(self, mock_settings):
        """Test _validate_model function."""
        _validate_mapping('foo', strict=True)
        mock_settings.get_index_mapping.assert_called_once_with('foo')
        mock_settings.get_index_mapping.side_effect = IOError()
        self.assertRaises(ImproperlyConfigured, _validate_mapping, 'foo', strict=True)
        # shouldn't raise error
        _validate_mapping('foo', strict=False)

    @mock.patch('elasticsearch_django.apps.settings')
    @mock.patch('elasticsearch_django.apps._validate_model')
    @mock.patch('elasticsearch_django.apps._validate_mapping')
    def test__validate_config(self, mock_mapping, mock_model, mock_settings):
        """Test _validate_model function."""
        mock_settings.get_index_names.return_value = ['foo']
        mock_settings.get_index_models.return_value = [TestModel]
        _validate_config()
        mock_mapping.assert_called_once_with('foo', strict=False)
        mock_model.assert_called_once_with(TestModel)

    @mock.patch('elasticsearch_django.apps.signals')
    @mock.patch('elasticsearch_django.apps.settings')
    def test__connect_signals(self, mock_settings, mock_signals):
        """Test the _connect_signals function."""
        # this should connect up the signals once, for TestModel
        mock_settings.get_index_names.return_value = ['foo']
        mock_settings.get_index_models.return_value = [TestModel]
        _connect_signals()
        mock_signals.post_save.connect.assert_called_once_with(
            _on_model_save,
            sender=TestModel,
            dispatch_uid='testmodel.post_save'
        )
        mock_signals.post_delete.connect.assert_called_once_with(
            _on_model_delete,
            sender=TestModel,
            dispatch_uid='testmodel.post_delete'
        )

    @mock.patch('elasticsearch_django.apps._update_search_index')
    def test__on_model_delete(self, mock_update):
        """Test the _on_model_delete function."""
        obj = SearchDocumentMixin()
        _on_model_delete(None, instance=obj)
        mock_update.assert_called_once_with(obj, 'delete', force=True)

    @mock.patch('elasticsearch_django.apps._update_search_index')
    def test__on_model_save(self, mock_update):
        """Test the _on_model_save function."""
        obj = SearchDocumentMixin()
        _on_model_save(None, instance=obj)
        mock_update.assert_called_once_with(obj, 'index')

    @mock.patch('elasticsearch_django.apps.logger')
    @mock.patch('elasticsearch_django.settings.get_model_indexes')
    @mock.patch.object(SearchDocumentMixin, 'update_search_index')
    def test__update_search_index(self, mock_update, mock_indexes, mock_logger):
        """Test the _update_search_index function."""
        mock_indexes.return_value = ['foo']
        obj = SearchDocumentMixin()
        _update_search_index(obj, 'delete')
        mock_update.assert_called_once_with('delete', index='foo', force=False)

        # if the update bombs, should still pass, and call logger
        mock_update.reset_mock()
        mock_update.side_effect = Exception()
        _update_search_index(obj, 'delete')
        mock_update.assert_called_once_with('delete', index='foo', force=False)
        mock_logger.exception.assert_called_once()

        # check that it calls for each configured index
        mock_update.reset_mock()
        mock_indexes.return_value = ['foo', 'bar']
        obj = SearchDocumentMixin()
        _update_search_index(obj, 'delete')
        mock_update.assert_has_calls([
            mock.call('delete', index='foo', force=False),
            mock.call('delete', index='bar', force=False)
        ])

        # confirm that it is **not** called if auto_sync is off
        mock_update.reset_mock()
        with mock.patch('elasticsearch_django.settings.get_setting') as mock_settings:
            mock_settings.return_value = False
            _update_search_index(obj, 'delete', force=True)
            mock_update.assert_not_called()



"""Create a search index."""
import logging

from elasticsearch.exceptions import TransportError

from . import BaseSearchCommand
from ...index import (
    delete_index,
    create_index,
    update_index
)

logger = logging.getLogger(__name__)


class Command(BaseSearchCommand):

    """Run the management command."""

    help = "Delete, create and update a new search index using the relevant mapping file."
    description = "Rebuild search index"

    def do_index_command(self, index, **options):
        """Rebuild search index."""
        if options['interactive']:
            logger.warning("This will permanently delete the index '%s'.", index)
            if not self._confirm_action():
                logger.warning("Aborting rebuild of index '%s' at user's request.", index)
                return

        try:
            delete = delete_index(index)
        except TransportError:
            delete = {}
            logger.info("Index %s does not exist, cannot be deleted.", index)
        create = create_index(index)
        update = update_index(index)

        return {
            'delete': delete,
            'create': create,
            'update': update
        }



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-11-29 11:35
from __future__ import unicode_literals

from django.db import migrations

from ..db.fields import JSONField


class Migration(migrations.Migration):

    dependencies = [
        ('elasticsearch_django', '0003_auto_20160926_2021'),
    ]

    operations = [
        migrations.AlterField(
            model_name='searchquery',
            name='hits',
            field=JSONField(help_text='The list of meta info for each of the query matches returned.'),
        ),
        migrations.AlterField(
            model_name='searchquery',
            name='query',
            field=JSONField(help_text='The raw ElasticSearch DSL query.'),
        ),
    ]



from unittest import mock

from django.test import TestCase
from django.test.utils import override_settings

from ..settings import (
    get_client,
    get_setting,
    get_settings,
    get_connection_string,
    get_index_config,
    get_index_names,
    get_index_models,
    get_model_indexes,
    get_document_models,
    get_document_model
)
from ..tests import TestModel


TEST_SETTINGS = {
    "connections": {
        "default": "https://foo",
        "backup": "https://bar"
    },
    "indexes": {
        "baz": {
            "models": ["elasticsearch_django.TestModel"]
        }
    },
    "settings": {
        "foo": "bar"
    }
}


class SettingsFunctionTests(TestCase):

    """Tests for the settings functions."""

    @mock.patch('elasticsearch_django.settings.get_connection_string')
    def test_get_client(self, mock_conn):
        """Test the get_client function."""
        mock_conn.return_value = "http://foo"
        client = get_client()
        self.assertEqual(len(client.transport.hosts), 1)
        self.assertEqual(client.transport.hosts[0]['host'], 'foo')

    @override_settings(SEARCH_SETTINGS=TEST_SETTINGS)
    def test_get_settings(self):
        """Test the get_settings method."""
        self.assertEqual(get_settings(), TEST_SETTINGS['settings'])

    @override_settings(SEARCH_SETTINGS=TEST_SETTINGS)
    def test_get_setting(self):
        """Test the get_setting method."""
        self.assertEqual(get_setting('foo'), 'bar')

    @override_settings(SEARCH_SETTINGS=TEST_SETTINGS)
    def test_get_connection_string(self):
        """Test the get_connection_string method."""
        self.assertEqual(get_connection_string(), TEST_SETTINGS['connections']['default'])
        self.assertEqual(get_connection_string('backup'), TEST_SETTINGS['connections']['backup'])

    @override_settings(SEARCH_SETTINGS=TEST_SETTINGS)
    def test_get_index_config(self):
        """Test the get_index_config method."""
        self.assertEqual(get_index_config('baz'), TEST_SETTINGS['indexes']['baz'])

    @override_settings(SEARCH_SETTINGS=TEST_SETTINGS)
    def test_get_index_names(self):
        """Test the get_index_names method."""
        self.assertEqual(get_index_names(), list(TEST_SETTINGS['indexes'].keys()))

    @override_settings(SEARCH_SETTINGS=TEST_SETTINGS)
    def test_get_index_models(self):
        """Test the get_index_models function."""
        from django.apps import apps
        models = get_index_models('baz')
        self.assertEqual(models, [apps.get_model('elasticsearch_django', 'TestModel')])

    @override_settings(SEARCH_SETTINGS=TEST_SETTINGS)
    def test_get_model_indexes(self):
        """Test the get_model_indexes function."""
        # TestModel is in the TEST_SETTINGS
        self.assertEqual(get_model_indexes(TestModel), ['baz'])
        # plain old object isn't in any indexes
        self.assertEqual(get_model_indexes(object), [])

    def test_get_index_mapping(self):
        """Test the get_index_mapping function."""
        # this interacts with the file system, not going to bother to test
        # as it just opens a file and loads in into a dict - there's no 'logic'
        pass

    @override_settings(SEARCH_SETTINGS=TEST_SETTINGS)
    def test_get_document_models(self):
        """Test the get_document_models function."""
        self.assertEqual(get_document_models(), {'baz.testmodel': TestModel})

    @override_settings(SEARCH_SETTINGS=TEST_SETTINGS)
    def test_get_document_model(self):
        """Test the get_document_model function."""
        self.assertEqual(get_document_model('baz', 'testmodel'), TestModel)



from django.test import TestCase

from ..settings import get_setting
from ..utils import disable_search_updates


class UtilsTests(TestCase):

    """elasticsearch_django.utils tests."""

    def test_disable_updates(self):

        self.assertTrue(get_setting('auto_sync'))

        with disable_search_updates():
            self.assertFalse(get_setting('auto_sync'))

        self.assertTrue(get_setting('auto_sync'))



# -*- coding: utf-8 -*-
# Generated by Django 1.11a1 on 2017-04-06 14:40
from __future__ import unicode_literals

from django.core.serializers.json import DjangoJSONEncoder
from django.db import migrations

from ..db.fields import JSONField


class Migration(migrations.Migration):

    dependencies = [
        ('elasticsearch_django', '0005_convert_JSONFields'),
    ]

    operations = [
        migrations.AlterField(
            model_name='searchquery',
            name='hits',
            field=JSONField(
                encoder=DjangoJSONEncoder,
                help_text='The list of meta info for each of the query matches returned.'
            ),
        ),
        migrations.AlterField(
            model_name='searchquery',
            name='query',
            field=JSONField(
                encoder=DjangoJSONEncoder,
                help_text='The raw ElasticSearch DSL query.'
            ),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('elasticsearch_django', '0002_searchquery_duration'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='searchquery',
            options={'verbose_name': 'Search query', 'verbose_name_plural': 'Search queries'},
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='SearchQuery',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('index', models.CharField(default='_all', help_text='The name of the ElasticSearch index(es) being queried.', max_length=100)),
                ('query', models.TextField(default='{}', help_text='The raw ElasticSearch DSL query.')),
                ('hits', models.TextField(default='{}', help_text='The list of meta info for each of the query matches returned.')),
                ('total_hits', models.IntegerField(default=0, help_text='Total number of matches found for the query (!= the hits returned).')),
                ('reference', models.CharField(default='', help_text='Custom reference used to identify and group related searches.', max_length=100, blank=True)),
                ('executed_at', models.DateTimeField(help_text='When the search was executed - set via execute() method.')),
                ('user', models.ForeignKey(related_name='search_queries', blank=True, to=settings.AUTH_USER_MODEL, help_text='The user who made the search query (nullable).', null=True, on_delete=models.SET_NULL)),
            ],
        ),
    ]



import simplejson as json  # simplejson supports Decimal serialization
import logging

from django.contrib import admin
from django.utils.safestring import mark_safe

from .models import SearchQuery

logger = logging.getLogger(__name__)


def pprint(data):
    """
    Returns an indented HTML pretty-print version of JSON.

    Take the event_payload JSON, indent it, order the keys and then
    present it as a <code> block. That's about as good as we can get
    until someone builds a custom syntax function.

    """
    pretty = json.dumps(
        data,
        sort_keys=True,
        indent=4,
        separators=(',', ': ')
    )
    html = pretty.replace(" ", "&nbsp;").replace("\n", "<br>")
    return mark_safe("<code>%s</code>" % html)


class SearchQueryAdmin(admin.ModelAdmin):

    list_display = (
        'user',
        'index',
        'total_hits',
        'returned_',
        'min_',
        'max_',
        'reference',
        'executed_at',
    )
    list_filter = (
        'index',
    )
    # excluding because we are using a pretty version instead
    exclude = ('hits', 'query', 'page')
    readonly_fields = (
        'user',
        'index',
        'total_hits',
        'returned_',
        'min_',
        'max_',
        'duration',
        'query_',
        'hits_',
        'executed_at',
    )

    def query_(self, instance):
        """Pretty version of query JSON."""
        return pprint(instance.query)

    def max_(self, instance):
        """Pretty version of max_score."""
        return '-' if instance.page_size == 0 else instance.max_score
    max_.short_description = "Max score"

    def min_(self, instance):
        """Pretty version of min_score."""
        return '-' if instance.page_size == 0 else instance.min_score
    min_.short_description = "Min score"

    def returned_(self, instance):
        """Number of hits returned in the page."""
        if instance.page_size == 0:
            return '-'
        else:
            return "%i - %i" % (instance.page_from, instance.page_to)

    returned_.short_description = "Page returned"

    def hits_(self, instance):
        """Pretty version of hits JSON."""
        return pprint(instance.hits)

admin.site.register(SearchQuery, SearchQueryAdmin)



from os import path, chdir, pardir
from setuptools import setup, find_packages

README = open(path.join(path.dirname(__file__), 'README.rst')).read()

# allow setup.py to be run from any path
chdir(path.normpath(path.join(path.abspath(__file__), pardir)))

setup(
    name="elasticsearch-django",
    version="5.1.1",
    packages=find_packages(),
    install_requires=[
        'Django>=1.11',
        'elasticsearch>=5',
        'elasticsearch-dsl>=5',
        'psycopg2-binary>=2.6',
        'simplejson>=3.8',
        'future'
    ],
    include_package_data=True,
    license='MIT',
    description='Elasticsearch Django app',
    long_description=README,
    url='https://github.com/yunojuno/elasticsearch-django',
    author='YunoJuno',
    author_email='code@yunojuno.com',
    maintainer='YunoJuno',
    maintainer_email='code@yunojuno.com',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



import logging

from django.apps import AppConfig
from django.core.exceptions import ImproperlyConfigured
from django.db.models import signals

from . import settings

logger = logging.getLogger(__name__)


class ElasticAppConfig(AppConfig):

    """AppConfig for Search3."""

    name = 'elasticsearch_django'
    verbose_name = "Elasticsearch"
    configs = []

    def ready(self):
        """Validate config and connect signals."""
        super(ElasticAppConfig, self).ready()
        _validate_config(settings.get_setting('strict_validation'))
        if settings.get_setting('auto_sync'):
            _connect_signals()
        else:
            logger.debug("SEARCH_AUTO_SYNC has been disabled.")


def _validate_config(strict=False):
    """Validate settings.SEARCH_SETTINGS."""
    for index in settings.get_index_names():
        _validate_mapping(index, strict=strict)
        for model in settings.get_index_models(index):
            _validate_model(model)


def _validate_mapping(index, strict=False):
    """Check that an index mapping JSON file exists."""
    try:
        settings.get_index_mapping(index)
    except IOError:
        if strict:
            raise ImproperlyConfigured("Index '%s' has no mapping file." % index)
        else:
            logger.warning("Index '%s' has no mapping, relying on ES instead.", index)


def _validate_model(model):
    """Check that a model configured for an index subclasses the required classes."""
    if not hasattr(model, 'as_search_document'):
        raise ImproperlyConfigured(
            "'%s' must implement `as_search_document`." % model
        )
    if not hasattr(model.objects, 'get_search_queryset'):
        raise ImproperlyConfigured(
            "'%s.objects must implement `get_search_queryset`." % model
        )


def _connect_signals():
    """Connect up post_save, post_delete signals for models."""
    for index in settings.get_index_names():
        for model in settings.get_index_models(index):
            dispatch_uid = '%s.post_save' % model._meta.model_name
            logger.debug("Connecting search index model sync signal: %s", dispatch_uid)
            signals.post_save.connect(_on_model_save, sender=model, dispatch_uid=dispatch_uid)
            dispatch_uid = '%s.post_delete' % model._meta.model_name
            logger.debug("Connecting search index model sync signal: %s", dispatch_uid)
            signals.post_delete.connect(_on_model_delete, sender=model, dispatch_uid=dispatch_uid)


def _on_model_save(sender, **kwargs):
    """Update documents in search index post_save."""
    _update_search_index(kwargs.get('instance'), 'index')


def _on_model_delete(sender, **kwargs):
    """
    Remove documents from search index post_delete.

    When deleting a document from the search index, always use force=True
    to ignore the in_search_queryset check - as by definition on a delete
    the object will no longer in exist in the database.

    """
    _update_search_index(kwargs.get('instance'), 'delete', force=True)


def _update_search_index(instance, action, force=False):
    """Process generic search index update actions."""
    # this allows us to turn off sync temporarily - e.g. when doing bulk updates
    if not settings.get_setting('auto_sync'):
        logger.debug("SEARCH_AUTO_SYNC disabled, ignoring update.")
        return
    for index in settings.get_model_indexes(instance.__class__):
        try:
            instance.update_search_index(action, index=index, force=force)
        except Exception:
            logger.exception("Error handling '%s' signal for %s", action, instance)



"""Delete a search index (and all documents therein)."""
import logging

from . import BaseSearchCommand
from ...index import delete_index

logger = logging.getLogger(__name__)


class Command(BaseSearchCommand):

    """Delete search index."""

    help = "Clears out the specified (or all) search index completely."
    description = "Delete search index"

    def do_index_command(self, index, **options):
        """Delete search index."""
        if options['interactive']:
            logger.warning("This will permanently delete the index '%s'.", index)
            if not self._confirm_action():
                logger.warning("Aborting deletion of index '%s' at user's request.", index)
                return
        return delete_index(index)
